using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;

namespace Utils.Log
{
    /// <summary>
    /// Log writing class
    /// </summary>   
    /// <remarks>This class is thread safe</remarks>
    public class Log
    {
        #region Variables
        /// <summary>
        /// Log file
        /// </summary>
        private FileInfo vFileToLog;
        /// <summary>
        /// Reference on the unique instance of Log
        /// </summary>
        private static readonly Log refLog = new Log();
        #endregion

        #region Constructeurs
        /// <summary>
        /// Default constructor
        /// </summary>
        /// <exception cref="Exception">Une erreur est survenue et il est ompossible de traiter les logs</exception>
        protected Log() { }
        /// <summary>
        /// Gives the reference of the object Log.
        /// </summary>
        /// <returns>A reference on the Log object</returns>
        public static Log GetInstance()
        {
            refLog.TestFile();
            return refLog;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Write a log without title ant the type if "Misc"
        /// </summary>
        /// <param name="_Msg">Log Message</param>
        public void Write(string _Msg)
        {
            try
            {
                Write(_Msg, "", LogType.Divers);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Write a log of "Misc" type
        /// </summary>
        /// <param name="_Title">Log title</param>
        /// <param name="_Msg">Log message</param>
        public void Write(string _Msg, string _Title)
        {
            try
            {
                Write(_Msg, _Title, LogType.Divers);
            }
            catch (Exception) { throw; }
        }
        /// <summary>
        /// Write a log
        /// </summary>
        /// <param name="_Type">Log type</param>
        /// <param name="_Title">Log title</param>
        /// <param name="_Msg">Log message</param>
        /// <exception cref="Exception">En cas d'erreur dans l'criture des logs</exception>
        public void Write(string _Msg, string _Title, LogType _Type)
        {
            lock (this)
            {
                #region Type
                string Type = _Type.ToString();
                int size = LogType.Information.ToString().Length - Type.Length;
                string wSpace = "";
                for (int i = 0; i < size; i++)
                    wSpace += " ";
                Type += wSpace;
                #endregion

                #region Title
                wSpace = "";
                size = 50 - _Title.Length;
                for (int i = 0; i < size; i++)
                    wSpace += " ";
                _Title += wSpace;
                #endregion

                #region Enlevers les Sparateurs
                _Msg = _Msg.Replace(CellSeparator, " ");
                _Msg = _Msg.Replace(LineSeparator, " ");
                #endregion

                #region Entre  logger
                string ToWrite = DateTime.Now.ToString() + CellSeparator
                    + Type + CellSeparator + _Title + CellSeparator + _Msg + CellSeparator;

                try
                {
                    StreamWriter sw = vFileToLog.AppendText();
                    sw.WriteLine(ToWrite);
                    sw.Close();
                }
                catch (ThreadStateException ex) { Log.Instance.Write(ex.Message, "Log.Write()", LogType.Erreur); }
                catch (ThreadAbortException) { }
                catch (Exception ex)
                {
                    throw new Exception("Erreur dans l'criture des logs" + ex.Message);
                }

                #endregion
            }
        }
        /// <summary>
        /// Check the size of the the log file
        /// </summary>
        private void TestFile()
        {

            try
            {
                lock (this)
                {
                    //Si le dosier n'existe pas
                    if (!Directory.Exists(CommandPath + @"\Log"))
                    {
                        string path = CommandPath + @"\Log";
                        Console.WriteLine(path);
                        Directory.CreateDirectory(path);
                    }

                    //Si le ficher est plus grand que 250k crer un nouveau
                    if (Directory.GetFiles(CommandPath + @"\Log\").Length != 0)
                    {
                        string[] NameOfFiles = Directory.GetFiles(CommandPath + @"\Log\");

                        FileInfo fi = new FileInfo(NameOfFiles[NameOfFiles.Length - 1]);

                        if (fi.Length > 250000)
                        {
                            string FilePath = CommandPath + @"\Log\Log_" + NameOfFiles.Length + ".wlg";
                            FileInfo newF = new FileInfo(FilePath);
                            newF.Create().Close();
                            StreamWriter sw = newF.AppendText();
                            sw.WriteLine(WLogTag);
                            sw.Close();
                            vFileToLog = newF;
                        }
                        else
                            vFileToLog = fi;
                    }

                    //cration d'un fichier s'il n'existe pas
                    else
                    {
                        string FilePath = CommandPath + @"\Log\Log.wlg";
                        FileInfo newF = new FileInfo(FilePath);
                        newF.Create().Close();
                        StreamWriter sw = newF.AppendText();
                        sw.WriteLine(WLogTag);
                        sw.WriteLine(DateTime.Now.ToString() + CellSeparator
                            + LogType.Information + CellSeparator + "First Log" + CellSeparator + "First Log" + CellSeparator);
                        sw.Close();
                        vFileToLog = newF;
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        #endregion

        #region Debug
        /// <summary>
        /// Gets the command line of this running application
        /// </summary>
        private string CommandLine { get { return Environment.CommandLine; } }
        /// <summary>
        /// Gets the execution path of this running application
        /// </summary>
        private string CommandPath
        {
            get
            {
                string Path = CommandLine;
                Path = Path.Remove(Path.LastIndexOf("\\"));
                Path = Path.Trim('\"');
                return Path;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Gets the cell separator
        /// </summary>
        public static string CellSeparator { get { return "|"; } }
        /// <summary>
        /// Gets the column separator
        /// </summary>
        public static string LineSeparator { get { return "\r\n"; } }
        /// <summary>
        /// Certification tag for a WLoGFile
        /// </summary>
        public static string WLogTag { get { return "<WLoG File>"; } }
        /// <summary>
        /// Gets the path of the file which contains the logs
        /// </summary>
        public string FileToLog { get { return vFileToLog.FullName; } }
        /// <summary>
        /// Gets the reference on the unique instance of Log
        /// </summary>
        public static Log Instance { get { return GetInstance(); } }
        #endregion
    }

    /// <summary>
    /// Donne le type d'entre de log
    /// </summary>
    public enum LogType
    {
        /// <summary>
        /// Divers
        /// </summary>
        Divers,
        /// <summary>
        /// Debug
        /// </summary>
        Debug,
        /// <summary>
        /// Erreur
        /// </summary>
        Erreur,
        /// <summary>
        /// Information
        /// </summary>
        Information,
        /// <summary>
        /// Information
        /// </summary>
        Rapport
    }
}
