using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.Log;
using Utils.Misc;
using WLog.Class;

namespace WLog.GUI
{
    /// <summary>
    /// Delegate for GUI refreshing
    /// </summary>
    internal delegate void RefreshGUIDelegate();
    /// <summary>
    /// Main frame
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// LogSet which contains all the logs
        /// </summary>
        private LogSet vLogSet = new LogSet();
        /// <summary>
        /// Indicates whether the HourComboBox is filling
        /// </summary>
        private bool vFillComboBox = true;
        /// <summary>
        /// Indicates the number of file changes
        /// </summary>
        private int vFileChanges = 0;
        /// <summary>
        /// Delegate for the GUI refreshing
        /// </summary>
        private RefreshGUIDelegate dRefreshGui;
        /// <summary>
        /// Options class
        /// </summary>
        private Options opt;
        /// <summary>
        /// Indicates whether an event changed the CheckBoxes
        /// </summary>
        private bool vEventCheck = false;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();

            tscb_Hours.SelectedIndexChanged += new EventHandler(Event_Hours_SelectedIndexChanged);
            vLogSet.FileChanged += new Utils.Log.Event.FileChangedEventHandler(Event_FileChanged);
            dRefreshGui = new RefreshGUIDelegate(RefreshGUI);
            if (!OptionContainer.Instance.Affected)
                OptionContainer.Instance.Options = new Options();
            opt = (Options)OptionContainer.Instance.Options;
            opt.OptionChanged += new EventHandler(Event_OptionChanged);
            SetOptions();
        }
        #endregion

        #region Events
        /// <summary>
        /// Open log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_OpenLog(object sender, EventArgs e)
        {
            DialogResult dr = ofd_OpenLog.ShowDialog();
            if (dr != DialogResult.OK) return;
            string _FilePath = ofd_OpenLog.FileName;

            OpenLog(_FilePath);
        }
        /// <summary>
        /// Close the logs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_CloseLog(object sender, EventArgs e)
        {
            this.dgv_Log.DataSource = null;
            this.dgv_Log.Enabled = false;
            this.vLogSet.Clear();
            this.tsmi_Search.Enabled = false;
            this.gb_Menu.Enabled = false;
            this.tsMenu_Open.Enabled = true;
            this.tsMenu_Close.Enabled = false;
            this.vFillComboBox = true;
            this.tssl_LogsNum.Text = "Entries: 0";
            this.tssl_Size.Text = "File size : 0 Ko";
            this.tsMenu_Guard.Enabled = false;
        }
        /// <summary>
        /// Search log by type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SearchByType(object sender, EventArgs e)
        {
            if (!vEventCheck)
            {
                vEventCheck = true;
                List<LogType> ltList = new List<LogType>();

                if (tsMenu_Debug.Checked) ltList.Add(LogType.Debug);
                if (tsMenu_Error.Checked) ltList.Add(LogType.Erreur);
                if (tsMenu_Information.Checked) ltList.Add(LogType.Information);
                if (tsMenu_Miscellaneous.Checked) ltList.Add(LogType.Divers);
                if (tsMenu_Report.Checked) ltList.Add(LogType.Rapport);

                #region Type searching
                if (sender == tsMenu_All)
                {
                    #region Menu & CheckBoxes
                    tsMenu_Debug.Checked = true;
                    tsMenu_Error.Checked = true;
                    tsMenu_Information.Checked = true;
                    tsMenu_Miscellaneous.Checked = true;
                    tsMenu_Report.Checked = true;

                    cb_Debug.Checked = true;
                    cb_Error.Checked = true;
                    cb_Information.Checked = true;
                    cb_Miscellaneous.Checked = true;
                    cb_Report.Checked = true;
                    #endregion

                    if (!ltList.Contains(LogType.Debug))
                        ltList.Add(LogType.Debug);
                    if (!ltList.Contains(LogType.Divers))
                        ltList.Add(LogType.Divers);
                    if (!ltList.Contains(LogType.Erreur))
                        ltList.Add(LogType.Erreur);
                    if (!ltList.Contains(LogType.Information))
                        ltList.Add(LogType.Information);
                    if (!ltList.Contains(LogType.Rapport))
                        ltList.Add(LogType.Rapport);
                }
                else if (sender == tsMenu_None)
                {
                    #region Menu & CheckBoxes
                    tsMenu_Debug.Checked = false;
                    tsMenu_Error.Checked = false;
                    tsMenu_Information.Checked = false;
                    tsMenu_Miscellaneous.Checked = false;
                    tsMenu_Report.Checked = false;

                    cb_Debug.Checked = false;
                    cb_Error.Checked = false;
                    cb_Information.Checked = false;
                    cb_Miscellaneous.Checked = false;
                    cb_Report.Checked = false;
                    #endregion

                    ltList.Clear();
                }
                else if (sender == tsMenu_Debug || sender == cb_Debug)
                {
                    if (!tsMenu_Debug.Checked)
                    {
                        tsMenu_Debug.Checked = true;
                        cb_Debug.Checked = true;
                        if (!ltList.Contains(LogType.Debug))
                            ltList.Add(LogType.Debug);
                    }
                    else
                    {
                        tsMenu_Debug.Checked = false;
                        cb_Debug.Checked = false;
                        ltList.Remove(LogType.Debug);
                    }
                }
                else if (sender == tsMenu_Error || sender == cb_Error)
                {
                    if (!tsMenu_Error.Checked)
                    {
                        tsMenu_Error.Checked = true;
                        cb_Error.Checked = true;
                        if (!ltList.Contains(LogType.Erreur))
                            ltList.Add(LogType.Erreur);
                    }
                    else
                    {
                        tsMenu_Error.Checked = false;
                        cb_Error.Checked = false;
                        ltList.Remove(LogType.Erreur);
                    }
                }
                else if (sender == tsMenu_Information || sender == cb_Information)
                {
                    if (!tsMenu_Information.Checked)
                    {
                        tsMenu_Information.Checked = true;
                        cb_Information.Checked = true;
                        if (!ltList.Contains(LogType.Information))
                            ltList.Add(LogType.Information);
                    }
                    else
                    {
                        tsMenu_Information.Checked = false;
                        cb_Information.Checked = false;
                        ltList.Remove(LogType.Information);
                    }
                }

                else if (sender == tsMenu_Miscellaneous || sender == cb_Miscellaneous)
                {
                    if (!tsMenu_Miscellaneous.Checked)
                    {
                        tsMenu_Miscellaneous.Checked = true;
                        cb_Miscellaneous.Checked = true;
                        if (ltList.Contains(LogType.Divers))
                            ltList.Add(LogType.Divers);
                    }
                    else
                    {
                        tsMenu_Miscellaneous.Checked = false;
                        cb_Miscellaneous.Checked = false;
                        ltList.Remove(LogType.Divers);
                    }
                }

                else if (sender == tsMenu_Report || sender == cb_Report)
                {
                    if (!tsMenu_Report.Checked)
                    {
                        tsMenu_Report.Checked = true;
                        cb_Report.Checked = true;
                        if (!ltList.Contains(LogType.Rapport))
                            ltList.Add(LogType.Rapport);
                    }
                    else
                    {
                        tsMenu_Report.Checked = false;
                        cb_Report.Checked = false;
                        ltList.Remove(LogType.Rapport);
                    }
                }
                #endregion

                LogSet _Result = new LogSet(this.vLogSet.FilePath);
                _Result = this.vLogSet.Search(ltList);
                this.RefreshDataGridView(_Result);
                this.RefreshHourComboBox();
                vEventCheck = false;
            }
        }
        /// <summary>
        /// Click on options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            Components.GUI.fr_OptionBox fr = new Components.GUI.fr_OptionBox();
            fr.OptionsObject = OptionContainer.Instance.Options;
            fr.ShowDialog();
        }
        /// <summary>
        /// About box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_About(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// The selected hour changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Hours_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!vFillComboBox)
            {
                LogSet _Result = new LogSet(this.vLogSet.FilePath);
                DateTime dt = DateTime.Parse((string)tscb_Hours.SelectedItem);
                _Result = this.vLogSet.Search(dt);
                RefreshDataGridView(_Result);
            }
            else
                vFillComboBox = false;
        }
        /// <summary>
        /// Indicates whether WLog will listen for change in the log file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_GuardFile(object sender, EventArgs e)
        {
            if (!tsMenu_Guard.Checked)
            {
                tsMenu_Guard.Checked = true;
                this.vLogSet.StartGuard();
            }
            else
            {
                tsMenu_Guard.Checked = false;
                this.vLogSet.StopGuard();
            }
        }
        /// <summary>
        /// Quit the application
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Indicates that the file was modified
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FileChanged(object sender, Utils.Log.Event.FileChangedEventArgs e)
        {
            this.Invoke(this.dRefreshGui);
        }
        /// <summary>
        /// Double click in the DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_DGV_DoubleClick(object sender, EventArgs e)
        {
            lock (this)
            {
                fr_LogReader fr = new fr_LogReader();
                fr.LogSet = this.vLogSet;
                fr.Index = this.dgv_Log.SelectedRows[0].Index;
                fr.ShowDialog();
            }
        }
        /// <summary>
        /// Options changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetOptions();
        }
        /// <summary>
        /// Click on All or None
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_AllNone(object sender, EventArgs e)
        {
            if (sender == btn_All)
            {
                cb_Debug.Checked = true;
                cb_Error.Checked = true;
                cb_Information.Checked = true;
                cb_Miscellaneous.Checked = true;
                cb_Report.Checked = true;
            }
            if (sender == btn_None)
            {
                cb_Debug.Checked = false;
                cb_Error.Checked = false;
                cb_Information.Checked = false;
                cb_Miscellaneous.Checked = false;
                cb_Report.Checked = false;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Refresh the GUI
        /// </summary>
        private void RefreshGUI()
        {
            tssl_FileChanged.Text = "File changes: " + (++vFileChanges);
            LogSet ls = new LogSet(this.vLogSet.FilePath);
            ls.Fill();
            this.vLogSet = ls;
            RefreshDataGridView(ls);
            RefreshHourComboBox();
        }
        /// <summary>
        /// Set up the DataGridView
        /// </summary>
        private void SetupDataGridView()
        {

            dgv_Log.Enabled = true;
            dgv_Log.Columns["Date"].DisplayIndex = 0;
            dgv_Log.Columns["Date"].DefaultCellStyle.Format = "dd/MM/yyyy [HH:mm:ss]";
            dgv_Log.Columns["Type"].DisplayIndex = 1;
            dgv_Log.Columns["Title"].DisplayIndex = 2;
            dgv_Log.Columns["Message"].DisplayIndex = 3;

        }
        /// <summary>
        /// Refresh the DataGridView with new data
        /// </summary>
        /// <param name="_LogSet"></param>
        private void RefreshDataGridView(LogSet _LogSet)
        {
            int _Index = dgv_Log.FirstDisplayedScrollingRowIndex;
            dgv_Log.DataSource = _LogSet.List;
            if (_Index < dgv_Log.Rows.Count && _Index > 0)
                dgv_Log.FirstDisplayedScrollingRowIndex = _Index;
            SetupDataGridView();
            RefreshLabels(_LogSet);
        }
        /// <summary>
        /// Refreshes the hours ComboBoxes
        /// </summary>
        private void RefreshHourComboBox()
        {
            this.vFillComboBox = true;
            this.tscb_Hours.Items.Clear();
            foreach (DateTime dt in vLogSet.ListOfDates)
            {
                this.tscb_Hours.ComboBox.Items.Add(dt.ToShortDateString());
            }
            if (this.tscb_Hours.ComboBox.Items.Count > 0)
                this.tscb_Hours.ComboBox.SelectedIndex = 0;
            this.vFillComboBox = false;
        }
        /// <summary>
        /// Refresh the labels
        /// </summary>
        private void RefreshLabels(LogSet _LogSet)
        {
            this.tssl_LogsNum.Text = "Entries: " + _LogSet.Count;
            this.tssl_Size.Text = "File size : " + _LogSet.FileSize.ToString("0.00") + " Ko";
        }
        /// <summary>
        /// Sets the options
        /// </summary>
        private void SetOptions()
        {
            #region Set DGV columns
            for (int i = 0; i < dgv_Log.Columns.Count; i++)
            {
                dgv_Log.Columns["Date"].Visible = opt.Date;
                dgv_Log.Columns["Type"].Visible = opt.TypeLog;
                dgv_Log.Columns["Title"].Visible = opt.Title;
                dgv_Log.Columns["Message"].Visible = opt.Message;
            }
            #endregion

            #region Menu
            if (!opt.Menu)
                tlp_General.RowStyles[0].Height = 0;
            else
                tlp_General.RowStyles[0].Height = Options.SIZE_OF_MENU;
            #endregion
        }
        /// <summary>
        /// Open logs and displays it
        /// </summary>
        /// <param name="_FilePath">Path of the log file</param>
        public void OpenLog(string _FilePath)
        {
            try
            {
                this.vLogSet.FilePath = _FilePath;
                this.vLogSet.Fill();
                this.dgv_Log.DataSource = vLogSet.List;
                this.SetupDataGridView();
                this.vFillComboBox = true;

                RefreshHourComboBox();

                this.tsmi_Search.Enabled = true;
                this.gb_Menu.Enabled = true;
                this.tsMenu_Open.Enabled = false;
                this.tsMenu_Close.Enabled = true;

                RefreshLabels(this.vLogSet);
                this.tsMenu_Guard.Enabled = true;
                vFileChanges = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible to fill the logs (" + ex.Message + ")", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion
    }
}